"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _log4js = _interopRequireDefault(require("log4js"));

var _IClass = _interopRequireDefault(require("../../support/base/IClass"));

/**
 * log4js 日志驱动
 *
 * @author : sunkeysun
 */
class Log4jsDriver extends _IClass.default {
  constructor(logConfig) {
    super();
    (0, _defineProperty2.default)(this, "_levels", ['debug', 'info', 'warn', 'error']);
    (0, _defineProperty2.default)(this, "_modes", ['console', 'single', 'dateFile', 'levelDateFile', 'levelDirDateFile']);
    (0, _defineProperty2.default)(this, "_layoutTypes", ['basic', 'messagePassThrough', 'dummy', 'pattern']);
    (0, _defineProperty2.default)(this, "_dateFileConfig", {
      type: 'dateFile',
      filename: '',
      alwaysIncludePattern: true,
      pattern: 'yyyyMMddhh.log'
    });
    (0, _defineProperty2.default)(this, "_levelFilterConfig", {
      type: 'logLevelFilter',
      appender: 'level',
      level: 'level'
    });
    (0, _defineProperty2.default)(this, "_defaultLayout", {
      type: 'pattern',
      pattern: '[%d] [%p] %m'
    });
    (0, _defineProperty2.default)(this, "_defaultConfig", {
      appenders: {},
      categories: {
        default: {
          appenders: ['debug'],
          level: 'debug'
        }
      },
      pm2: true,
      pm2InstanceVar: 'NODE_APP_INSTANCE',
      replaceConsole: true
    });

    if (!~this._modes.indexOf(logConfig.mode)) {
      throw new Error(`log config mode is invalid! must value of ${JSON.stringify(this._modes)}`);
    }

    const logPath = logConfig.logPath;
    const layout = { ...this._defaultLayout,
      ..._.get(logConfig, 'layout', {})
    };

    const targetConfig = _.extend({}, this._defaultConfig, _.pick(_.keys(this._defaultConfig)));

    if (!~this._layoutTypes.indexOf(layout.type)) {
      _log4js.default.addLayout(layout.type, layout.format);
    }

    if (logConfig.mode === 'console') {
      targetConfig.appenders = {
        default: {
          type: 'console',
          layout
        }
      };
      targetConfig.categories = {
        default: {
          appenders: ['default'],
          level: 'debug'
        }
      };
    } else if (logConfig.mode === 'single') {
      targetConfig.appenders = {
        default: {
          type: 'file',
          filename: `${logPath}/log.txt`,
          layout
        }
      };
      targetConfig.categories = {
        default: {
          appenders: ['default'],
          level: 'debug'
        }
      };
    } else if (logConfig.mode === 'dateFile') {
      targetConfig.appenders = {
        default: { ...pattern,
          pattern: _.get(logConfig, 'pattern', this._dateFileConfig.pattern),
          filename: `${logPath}/log-`,
          layout
        }
      };
      targetConfig.categories = {
        default: {
          appenders: ['default'],
          level: 'debug'
        }
      };
    } else {
      targetConfig.appenders = _.reduce(this._levels, (result, level) => {
        let filename = '';

        if (logConfig.mode === 'levelDateFile') {
          filename = `${logPath}/${level}-`;
        } else if (logConfig.mode === 'levelDirDateFile') {
          filename = `${logPath}/${level}/`;
        }

        return { ...result,
          [level]: { ...this._dateFileConfig,
            pattern: _.get(logConfig, 'pattern', this._dateFileConfig.pattern),
            filename: filename,
            layout
          }
        };
      }, targetConfig.appenders);
      targetConfig.categories = _.reduce(this._levels, (result, level) => {
        return { ...result,
          [level]: {
            level,
            appenders: [`${level}`]
          }
        };
      }, targetConfig.categories);
    }

    _log4js.default.configure(targetConfig);

    _.each(this._levels, level => {
      this[level] = msg => {
        _log4js.default.getLogger(level)[level](msg);
      };
    });
  }

}

exports.default = Log4jsDriver;