"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _IClass = _interopRequireDefault(require("../support/base/IClass"));

/**
 * 路由类
 *
 * @author : sunkeysun
 */
class Route extends _IClass.default {
  get path() {
    return this._path;
  }

  get locals() {
    return this._locals;
  }

  get middlewares() {
    return this._middlewares;
  }

  get handler() {
    return this._handler;
  }

  constructor({
    path: routePath,
    handler,
    middlewares,
    locals
  }) {
    super();
    (0, _defineProperty2.default)(this, "_handler", null);
    (0, _defineProperty2.default)(this, "_path", '');
    (0, _defineProperty2.default)(this, "_locals", {});
    (0, _defineProperty2.default)(this, "_middlewares", []);
    this._path = routePath;
    this._handler = handler;
    this._middlewares = middlewares;
    this._locals = locals;

    if (_.isString(handler)) {
      if (!~handler.indexOf('@')) {
        throw new Error(`route handler string must like {controller}@{action}, now is ${handler}`);
      }

      const handlerArr = handler.split('@');
      const controllerName = handlerArr[0];
      const actionName = handlerArr[1];
      const modulePath = `${app().controllerPath}/${controllerName}`;

      const Controller = require(modulePath).default;

      this._handler = {
        controller: Controller,
        action: actionName
      };
    }
  }

  handle(request, response) {
    if (_.isFunction(this._handler)) {
      return this._handler.apply(null, [request, response]);
    } else {
      const Controller = this._handler.controller;
      const actionName = this._handler.action;
      const controllerInstance = new Controller(request, response);

      if (!_.isFunction(controllerInstance[actionName])) {
        throw new Error(`action (${actionName}) function not defined`);
      }

      return controllerInstance[actionName]();
    }
  }

}

exports.default = Route;