"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _path = _interopRequireDefault(require("path"));

var _koaRouter = _interopRequireDefault(require("koa-router"));

var _IClass = _interopRequireDefault(require("../support/base/IClass"));

var _Route = _interopRequireDefault(require("./Route"));

var _Request = _interopRequireDefault(require("../http/Request"));

var _Response = _interopRequireDefault(require("../http/Response"));

var _helpers = require("../support/helpers");

/**
 * 路由处理器
 *
 * @author : sunkeysun
 */
const coreRouter = new _koaRouter.default();
const routes = [];

class Router extends _IClass.default {
  constructor(...args) {
    super(...args);
    (0, _defineProperty2.default)(this, "_inited", false);
    (0, _defineProperty2.default)(this, "_routePath", '');
    (0, _defineProperty2.default)(this, "_options", {
      middlewares: [],
      locals: {}
    });
  }

  get routePath() {
    return this._routePath;
  }

  set routePath(routePath) {
    this._routePath = routePath;
  }

  set options(options) {
    this._options = options;
  }

  get(...args) {
    args.unshift('get');
    return this._handle(...args);
  }

  post(...args) {
    args.unshift('post');
    return this._handle(...args);
  }

  put(...args) {
    args.unshift('put');
    return this._handle(...args);
  }

  delete(...args) {
    args.unshift('delete');
    return this._handle(...args);
  }

  patch(...args) {
    args.unshift('patch');
    return this._handle(...args);
  }

  resource(resource, controller, options = {}) {
    const type = 'resource';
    this.post(`${resource}`, `${controller}@add`, { ...options,
      type
    });
    this.delete(`${resource}/:id`, `${controller}@remove`, { ...options,
      type
    });
    this.put(`${resource}/:id`, `${controller}@update`, { ...options,
      type
    });
    this.get(`${resource}`, `${controller}@list`, { ...options,
      type
    });
    this.get(`${resource}/:id`, `${controller}@query`, { ...options,
      type
    });
  }

  group(prefix, handler, {
    middlewares = [],
    locals = {}
  } = {}) {
    const prePrefix = this._routePath;
    const routePath = `${prePrefix}${prefix}`;

    const groupRouter = _.cloneDeep(this);

    groupRouter.routePath = routePath;
    groupRouter.options = {
      middlewares: _.uniq([...groupRouter._options.middlewares, ...middlewares]),
      locals: { ...groupRouter._options.locals,
        ...locals
      }
    };
    handler(groupRouter);
    return groupRouter;
  }

  _initRoutes() {
    // 资源路由降低匹配优先级
    _.each(routes, route => {
      const routeArgs = [route.path, ...route.middlewares, route.handler];
      coreRouter[route.method].apply(coreRouter, routeArgs);

      if (route.path !== '/' && route.path.endsWith('/')) {
        const routeArgsEx = [_.trimEnd(route.path, '/'), ...route.middlewares, route.handler];
        coreRouter[route.method].apply(coreRouter, routeArgs);
      }
    });

    return true;
  }

  routes() {
    if (this._inited) {
      return coreRouter.routes();
    }

    this._initRoutes();

    this._inited = true;
    return coreRouter.routes();
  }

  _getMiddlewareHandlers(middlewares = []) {
    const middlewareHandlers = _.map(middlewares, Middleware => {
      return (0, _helpers.normalizeMiddleware)(Middleware.handle.bind(Middleware));
    });

    return middlewareHandlers;
  }

  _handle(method, routePath, handler, {
    middlewares = [],
    locals = {},
    type = 'normal'
  } = {}) {
    const prefix = _.get(this, 'routePath', '');

    if (_.isArray(routePath)) {
      routePath = _.map(routePath, routePathItem => {
        return _path.default.posix.normalize(`${prefix}${routePathItem}`);
      });
    } else {
      routePath = _path.default.posix.normalize(`${prefix}${routePath}`);
    }

    const allLocals = { ...this._options.locals,
      ...locals
    };

    const allMiddlewares = _.uniq([...this._options.middlewares, ...middlewares]);

    const middlewareHandlers = this._getMiddlewareHandlers(allMiddlewares);

    const route = new _Route.default({
      handler,
      path: routePath,
      middlewares: middlewareHandlers,
      locals: allLocals
    });

    const routeHandler = async (ctx, next) => {
      try {
        await route.handle(ctx.iRequest, ctx.iResponse);
      } catch (e) {
        const event = app().event('route-error');

        if (app().listenerCount(event) > 0) {
          app().emit(event, e, ctx.iRequest, ctx.iResponse);
        }

        app().logger.error(e, 'route');
        return ctx.iResponse.render500(e);
      }
    };

    const routeMiddleware = async (ctx, next) => {
      ctx.iRequest.route = route;
      await next();
    };

    routes.push({
      type,
      method,
      path: route.path,
      middlewares: [routeMiddleware, ...route.middlewares],
      handler: routeHandler
    });
    return this;
  }

}

exports.default = Router;