"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _Provider = _interopRequireDefault(require("../support/base/Provider"));

var _koaSession = _interopRequireDefault(require("koa-session"));

var _RedisStore = _interopRequireDefault(require("./stores/RedisStore"));

/**
 * session 服务提供者
 *
 * @author : sunkeysun
 */
class SessionProvider extends _Provider.default {
  register() {
    const sessionConfig = app().config('session');
    let store = null;

    switch (sessionConfig.store) {
      case 'redis':
        const redisConfig = _.get(sessionConfig, 'config.redis');

        store = new _RedisStore.default(redisConfig, {
          timeout: sessionConfig.timeout
        });
        break;

      default:
        store = null;
        break;
    }

    let targetConfig = _.pick(sessionConfig, ['key', 'maxAge', 'overwrite', 'httpOnly', 'signed', 'rolling', 'renew', 'secure', 'sameSite', 'domain', 'autoCommit', 'decode', 'encode']);

    if (!!store) {
      targetConfig = _.extend(targetConfig, {
        store
      });
    }

    return _.partial(_koaSession.default, targetConfig);
  }

}

exports.default = SessionProvider;