"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _IClass = _interopRequireDefault(require("../../support/base/IClass"));

var _RedisDriver = _interopRequireDefault(require("../../support/drivers/RedisDriver"));

var _helpers = require("../helpers");

/**
 * redis session 存储驱动
 *
 * @author : sunkeysun
 */
class RedisStore extends _IClass.default {
  constructor(redisConfig, {
    timeout = 200
  }) {
    super();
    (0, _defineProperty2.default)(this, "_redis", null);
    (0, _defineProperty2.default)(this, "_timeout", void 0);
    this._redis = new _RedisDriver.default(redisConfig);
    this._timeout = timeout;
  }

  async get(sid, maxAge, {
    rolling
  }) {
    try {
      const data = await this._redis.get(sid).timeout(this._timeout);
      const resultData = (0, _helpers.unserialize)(data);
      return resultData;
    } catch (e) {
      app().emit(app().event('session-error'), e, 'get');
      return false;
    }
  }

  async set(sid, sess, maxAge, {
    changed,
    rolling
  }) {
    try {
      return await this._redis.set(sid, (0, _helpers.serialize)(sess), 'EX', Math.floor(maxAge / 1000)).timeout(this._timeout);
    } catch (e) {
      app().emit(app().event('session-error'), e, 'set');
      return false;
    }
  }

  async destroy(sid) {
    try {
      return await this._redis.del(sid).timeout(this._timeout);
    } catch (e) {
      app().emit(app().event('session-error'), e, 'destroy');
    }
  }

}

exports.default = RedisStore;