"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _ioredis = _interopRequireDefault(require("ioredis"));

var _Driver = _interopRequireDefault(require("../base/Driver"));

/**
 * redis 驱动
 *
 * @author : sunkeysun
 */
class RedisDriver extends _Driver.default {
  constructor(redisConfig) {
    super();
    (0, _defineProperty2.default)(this, "_redis", null);

    const options = _.get(redisConfig, 'options', {});

    if (redisConfig.mode === 'cluster') {
      const clusterOptions = _.get(redisConfig, 'clusterOptions', {});

      clusterOptions.redisOptions = options;
      this._redis = new _ioredis.default.Cluster(redisConfig.servers, clusterOptions);
    } else if (redisConfig.mode === 'single') {
      const targetServer = _.get(redisConfig.servers, '0', {});

      this._redis = new _ioredis.default({ ...options,
        ...targetServer
      });
    }

    if (!this._redis) {
      return {};
    }

    this._redis.on('connect', () => {
      app().emit(app().event('redis-connect'), redisConfig);
      app().logger.info(`redis connect [${JSON.stringify(redisConfig)}]`, 'redis');
    });

    this._redis.on('ready', () => {
      app().emit(app().event('redis-ready'), redisConfig);
      app().logger.info(`redis ready [${JSON.stringify(redisConfig)}]`, 'redis');
    });

    this._redis.on('error', e => {
      app().emit(app().event('redis-error'), e, redisConfig);
      app().logger.error(`redis error [${JSON.stringify(redisConfig)}] - ${e}`, 'redis');
    });

    return this._redis;
  }

}

exports.default = RedisDriver;